import * as React from 'react';
declare type RenderFunc<T> = (item: T, index: number) => React.ReactNode;
export interface RelativeScroll {
    itemIndex: number;
    relativeTop: number;
}
export interface ScrollInfo {
    scrollTop: number;
    startItemTop: number;
    startIndex: number;
}
export interface ListProps<T> extends React.HTMLAttributes<any> {
    children: RenderFunc<T>;
    dataSource: T[];
    height?: number;
    itemHeight?: number;
    itemKey: string;
    component?: string | React.FC<any> | React.ComponentClass<any>;
    disabled?: boolean;
}
interface ListState<T> {
    status: 'NONE' | 'MEASURE_START' | 'MEASURE_DONE';
    scrollTop: number | null;
    /** Located item index */
    itemIndex: number;
    /** Located item bind its height percentage with the `scrollTop` */
    itemOffsetPtg: number;
    startIndex: number;
    endIndex: number;
    /**
     * Calculated by `scrollTop`.
     * We cache in the state since if `dataSource` length change,
     * we need revert back to the located item index.
     */
    startItemTop: number;
}
/**
 * We use class component here since typescript can not support generic in function component
 *
 * Virtual list display logic:
 * 1. scroll / initialize trigger measure
 * 2. Get location item of current `scrollTop`
 * 3. [Render] Render visible items
 * 4. Get all the visible items height
 * 5. [Render] Update top item `margin-top` to fit the position
 *
 * Algorithm:
 * We split scroll bar into equal slice. An item with whatever height occupy the same range slice.
 * When `scrollTop` change,
 * it will calculate the item percentage position and move item to the position.
 * Then calculate other item position base on the located item.
 *
 * Concept:
 *
 * # located item
 * The base position item which other items position calculate base on.
 */
declare class List<T> extends React.Component<ListProps<T>, ListState<T>> {
    static defaultProps: {
        itemHeight: number;
        dataSource: any[];
    };
    state: ListState<T>;
    listRef: React.RefObject<HTMLElement>;
    itemElements: {
        [index: number]: HTMLElement;
    };
    itemElementHeights: {
        [index: number]: number;
    };
    /**
     * Always point to the latest props if `disabled` is `false`
     */
    cachedProps: Partial<ListProps<T>>;
    /**
     * Lock scroll process with `onScroll` event.
     * This is used for `dataSource` length change and `scrollTop` restore
     */
    lockScroll: boolean;
    /**
     * Phase 1: Initial should sync with default scroll top
     */
    componentDidMount(): void;
    /**
     * Phase 4: Record used item height
     * Phase 5: Trigger re-render to use correct position
     */
    componentDidUpdate(): void;
    /**
     * Phase 2: Trigger render since we should re-calculate current position.
     */
    onScroll: () => void;
    getIndexKey: (index: number, props?: Partial<ListProps<T>>) => any;
    getItemKey: (item: T, props?: Partial<ListProps<T>>) => any;
    /**
     * Collect current rendered dom element item heights
     */
    collectItemHeights: () => void;
    scrollTo(arg: number | RelativeScroll): void;
    /**
     * Phase 4: Render item and get all the visible items height
     */
    renderChildren: (list: T[], startIndex: number, renderFunc: RenderFunc<T>) => React.ReactElement<any, string | ((props: any) => React.ReactElement<any, string | any | (new (props: any) => React.Component<any, any, any>)>) | (new (props: any) => React.Component<any, any, any>)>[];
    render(): JSX.Element;
}
export default List;
